@@ -46,8 +46,8 @@ public:
     virtual void pack(OutputFile *fo) override;
     virtual void unpack(OutputFile *fo) override;
 
-    virtual bool canPack() override;
-    virtual int canUnpack() override;
+    virtual tribool canPack() override;
+    virtual tribool canUnpack() override;
 
     // unpacker capabilities
     virtual bool canUnpackVersion(int version) const override {
