@@ -51,7 +51,7 @@
 #include "xspan_impl.h"
 
 #ifdef XSPAN_NAMESPACE_NAME
-// help constructor to distinguish between number of elements and bytes
+// types to help the constructor to distinguish between number of elements and bytes
 using XSPAN_NAMESPACE_NAME::XSpanCount;
 using XSPAN_NAMESPACE_NAME::XSpanSizeInBytes;
 // actual classes
@@ -59,7 +59,7 @@ using XSPAN_NAMESPACE_NAME::Ptr;
 using XSPAN_NAMESPACE_NAME::PtrOrSpan;
 using XSPAN_NAMESPACE_NAME::PtrOrSpanOrNull;
 using XSPAN_NAMESPACE_NAME::Span;
-// util
+// support functions
 using XSPAN_NAMESPACE_NAME::raw_bytes;       // overloaded for all classes
 using XSPAN_NAMESPACE_NAME::raw_index_bytes; // overloaded for all classes
 #endif
@@ -95,25 +95,35 @@ using XSPAN_NAMESPACE_NAME::raw_index_bytes; // overloaded for all classes
 #define XSPAN_P_VAR(type, var, first, ...) XSPAN_P(type) var((first), ##__VA_ARGS__)
 #define XSPAN_S_VAR(type, var, first, ...) XSPAN_S(type) var((first), ##__VA_ARGS__)
 
+// cast to a different type (creates a new value)
+#define XSPAN_TYPE_CAST(type, x) ((x).type_cast<type>())
+// poison a pointer: point to a non-null invalid address
+#define XSPAN_INVALIDATE(x)      ((x).invalidate())
+
 #elif WITH_XSPAN >= 1
 
 // unchecked - just a no-op pointer wrapper, no extra functionality
 
 // types
-#define XSPAN_0(type) Ptr<type>
-#define XSPAN_P(type) Ptr<type>
-#define XSPAN_S(type) Ptr<type>
+#define XSPAN_0(type)                      Ptr<type>
+#define XSPAN_P(type)                      Ptr<type>
+#define XSPAN_S(type)                      Ptr<type>
 
 // create a value
-#define XSPAN_0_MAKE(type, first, ...) (XSPAN_0(type)((first)))
-#define XSPAN_P_MAKE(type, first, ...) (XSPAN_P(type)((first)))
-#define XSPAN_S_MAKE(type, first, ...) (XSPAN_S(type)((first)))
+#define XSPAN_0_MAKE(type, first, ...)     (XSPAN_0(type)((first)))
+#define XSPAN_P_MAKE(type, first, ...)     (XSPAN_P(type)((first)))
+#define XSPAN_S_MAKE(type, first, ...)     (XSPAN_S(type)((first)))
 
 // define a variable
 #define XSPAN_0_VAR(type, var, first, ...) XSPAN_0(type) var((first))
 #define XSPAN_P_VAR(type, var, first, ...) XSPAN_P(type) var((first))
 #define XSPAN_S_VAR(type, var, first, ...) XSPAN_S(type) var((first))
 
+// cast to a different type (creates a new value)
+#define XSPAN_TYPE_CAST(type, x)           ((x).type_cast<type>())
+// poison a pointer: point to a non-null invalid address
+#define XSPAN_INVALIDATE(x)                ((x).invalidate())
+
 #else // WITH_XSPAN
 
 // unchecked regular pointers
@@ -133,20 +143,25 @@ inline R *xspan_make_helper__(R * /*dummy*/, MemBuffer &mb) noexcept {
 }
 
 // types
-#define XSPAN_0(type) type *
-#define XSPAN_P(type) type *
-#define XSPAN_S(type) type *
+#define XSPAN_0(type)                      type *
+#define XSPAN_P(type)                      type *
+#define XSPAN_S(type)                      type *
 
 // create a value
-#define XSPAN_0_MAKE(type, first, ...) (xspan_make_helper__((type *) nullptr, (first)))
-#define XSPAN_P_MAKE(type, first, ...) (xspan_make_helper__((type *) nullptr, (first)))
-#define XSPAN_S_MAKE(type, first, ...) (xspan_make_helper__((type *) nullptr, (first)))
+#define XSPAN_0_MAKE(type, first, ...)     (xspan_make_helper__((type *) nullptr, (first)))
+#define XSPAN_P_MAKE(type, first, ...)     (xspan_make_helper__((type *) nullptr, (first)))
+#define XSPAN_S_MAKE(type, first, ...)     (xspan_make_helper__((type *) nullptr, (first)))
 
 // define a variable
 #define XSPAN_0_VAR(type, var, first, ...) type *var = XSPAN_0_MAKE(type, (first))
 #define XSPAN_P_VAR(type, var, first, ...) type *var = XSPAN_P_MAKE(type, (first))
 #define XSPAN_S_VAR(type, var, first, ...) type *var = XSPAN_S_MAKE(type, (first))
 
+// cast to a different type (creates a new value)
+#define XSPAN_TYPE_CAST(type, x)           (reinterpret_cast<type *>(x))
+// poison a pointer: point to a non-null invalid address
+#define XSPAN_INVALIDATE(x)                ptr_invalidate_and_poison(x)
+
 #endif // WITH_XSPAN
 
 /*************************************************************************
@@ -155,17 +170,21 @@ inline R *xspan_make_helper__(R * /*dummy*/, MemBuffer &mb) noexcept {
 
 #if 1
 // types
-#define SPAN_0 XSPAN_0
-#define SPAN_P XSPAN_P
-#define SPAN_S XSPAN_S
+#define SPAN_0          XSPAN_0
+#define SPAN_P          XSPAN_P
+#define SPAN_S          XSPAN_S
 // create a value
-#define SPAN_0_MAKE XSPAN_0_MAKE
-#define SPAN_P_MAKE XSPAN_P_MAKE
-#define SPAN_S_MAKE XSPAN_S_MAKE
+#define SPAN_0_MAKE     XSPAN_0_MAKE
+#define SPAN_P_MAKE     XSPAN_P_MAKE
+#define SPAN_S_MAKE     XSPAN_S_MAKE
 // define a variable
-#define SPAN_0_VAR XSPAN_0_VAR
-#define SPAN_P_VAR XSPAN_P_VAR
-#define SPAN_S_VAR XSPAN_S_VAR
+#define SPAN_0_VAR      XSPAN_0_VAR
+#define SPAN_P_VAR      XSPAN_P_VAR
+#define SPAN_S_VAR      XSPAN_S_VAR
+// cast to a different type (creates a new value)
+#define SPAN_TYPE_CAST  XSPAN_TYPE_CAST
+// poison a pointer: point to a non-null invalid address
+#define SPAN_INVALIDATE XSPAN_INVALIDATE
 #endif
 
 /* vim:set ts=4 sw=4 et: */
